<?php declare(strict_types=1);

namespace GirosGiroCheckout6;

use Doctrine\DBAL\Connection;
use Dompdf\Exception;
use GirosGiroCheckout6\Service\PaymentGcbluecode;
use GirosGiroCheckout6\Service\PaymentGccreditcard;
use GirosGiroCheckout6\Service\PaymentGcdirectdebit;
use GirosGiroCheckout6\Service\PaymentGceps;
use GirosGiroCheckout6\Service\PaymentGcgiropay;
use GirosGiroCheckout6\Service\PaymentGcideal;
use GirosGiroCheckout6\Service\PaymentGcmaestro;
use GirosGiroCheckout6\Service\PaymentGcpaydirekt;
use GirosGiroCheckout6\Service\PaymentGcsofortuw;
use GirosGiroCheckout6\Helper\AbstractPaymentProxy;
use GirosGiroCheckout6\Helper\MediaInstaller;
use Shopware\Core\Framework\Plugin;
use Shopware\Core\Framework\Context;
use Shopware\Core\Content\Media\MediaEntity;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\Plugin\Context\UpdateContext;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;
use Shopware\Core\Content\Media\MediaService;

if (!defined('GIROCONNECT_CONFIGURE_SEND_EMAIL')) {
  define('GIROCONNECT_CONFIGURE_SEND_EMAIL', TRUE);
}

// IMPORTANT INCLUDES
$pluginDir = dirname(__FILE__);
$helpersDir = $pluginDir . '/Helper';
require_once __DIR__ . '/../vendor/autoload.php';

if (is_file($helpersDir . '/AbstractPaymentProxy.php')) {
  include_once $helpersDir . '/AbstractPaymentProxy.php';
}
if (is_file($helpersDir . '/CreditcardPaymentProxy.php')) {
  include_once $helpersDir . '/CreditcardPaymentProxy.php';
}

if (class_exists('AbstractPaymentProxy')) {
  AbstractPaymentProxy::includeGiroconnectLibrary();
}

class GirosGiroCheckout6 extends Plugin
{
  const PLUGIN_NAME = 'GirosGiroCheckout6';
  const PLUGIN_VERSION = '4.0.8';
  const TABLE_USER_PAYMENT_SETTINGS = 'gc_user_pay_giroconnect';
  const TABLE_GC_TRANSACTIONS = 'gc_user_pay_giroconnect_trx';
  const PAYMENT_NAME_GIROPAY = 'gc_giropay';
  const PAYMENT_NAME_DIRECTDEBIT = 'gc_directdebit';
  const PAYMENT_NAME_CREDITCARD = 'gc_creditcard';
  const PAYMENT_NAME_IDEAL = 'gc_ideal';
  const PAYMENT_NAME_EPS = 'gc_eps';
  const PAYMENT_NAME_PAYDIREKT = 'gc_paydirekt';
  const PAYMENT_NAME_SOFORTUW = 'gc_sofortuw';
  const PAYMENT_NAME_MAESTRO = 'gc_maestro';


  /**
   * @param ContainerBuilder $container
   */
  public function build($container): void
  {
    parent::build($container);
    $this->container = $container;
    $loader = new XmlFileLoader($container, new FileLocator(__DIR__ . '/Resources/config'));
    $loader->load('services.xml');
  }

  public function boot(): void
  {
    parent::boot();
  }

  public function install(InstallContext $context): void
  {
    parent::install($context);
    $this->addPaymentMethod($context->getContext());
  }

  public function update(UpdateContext $context): void
  {
    parent::update($context);
  }

  public function uninstall(UninstallContext $context): void
  {
    // Only set the payment method to inactive when uninstalling. Removing the payment method would
    // cause data consistency issues, since the payment method might have been used in several orders
    $this->setPaymentMethodIsActive(false, $context->getContext());

    //Drop media files.
    $connection = $this->container->get(Connection::class);
    $connection->executeUpdate(\sprintf("DELETE FROM media_folder WHERE name = 'GirosGiroCheckout6 Logos'"));
    $connection->executeUpdate(\sprintf("DELETE FROM media_default_folder WHERE entity = 'gsgirocheckout'"));
  }

  public function activate(ActivateContext $context): void
  {
    parent::activate($context);
    $mediaInstaler = $this->container->get('GirosGiroCheckout6\Helper\MediaInstaller');
    $mediaInstaler->activate($context);
    //Active all payment methods. (Default: none)
    //$this->setPaymentMethodIsActive(true, $context->getContext());
  }

  public function deactivate(DeactivateContext $context): void
  {
    $this->setPaymentMethodIsActive(false, $context->getContext());
    $mediaInstaler = $this->container->get('GirosGiroCheckout6\Helper\MediaInstaller');
    $mediaInstaler->deactivate($context);
    parent::deactivate($context);
  }

  private function addPaymentMethod(Context $context): void
  {
    $pluginIdProvider = $this->container->get(PluginIdProvider::class);
    $pluginId = $pluginIdProvider->getPluginIdByBaseClass(get_class($this), $context);
    $paymentRepository = $this->container->get('payment_method.repository');

    //CREDIT CARD
    $options = [
      'handlerIdentifier' => PaymentGccreditcard::class,
      'id' => $this->getPaymentMethodId(PaymentGccreditcard::class, $context),
      'name' => 'Kreditkarte',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'Kreditkarte',
          'description' => 'Im nächsten Schritt werden Sie aufgefordert, Ihre Kreditkartendaten einzugeben. Wenn Ihre Kreditkarte für 3D-Secure (Verified by Visa bzw. MasterCard SecureCode) freigeschaltet ist, werden Sie während des Bezahlprozesses zur Sicherheitsabfrage Ihrer Bank weitergeleitet.',
        ],
        'en-GB' => [
          'name' => 'Credit Card',
          'description' => 'On the following secure website, you are asked to enter your credit card data. If your credit card is enabled for 3D Secure (Verified by Visa or MasterCard Secure Code), additional information will be passed for a security check at your bank.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGccreditcard::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //DIRECT DEBIT
    $options = [
      'handlerIdentifier' => PaymentGcdirectdebit::class,
      'id' => $this->getPaymentMethodId(PaymentGcdirectdebit::class, $context),
      'name' => 'Lastschrift',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'Lastschrift',
          'description' => 'Hiermit ermächtigen Sie uns, den fälligen Betrag von Ihrem Konto mittels Lastschrift einzuziehen.',
        ],
        'en-GB' => [
          'name' => 'Direct Debit',
          'description' => 'Hereby you authorize us to charge your current account with the invoiced amount by direct debit.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcdirectdebit::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //GIROPAY
    $options = [
      'handlerIdentifier' => PaymentGcgiropay::class,
      'id' => $this->getPaymentMethodId(PaymentGcgiropay::class, $context),
      'name' => 'giropay - Online überweisen',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'giropay - Online überweisen',
          'description' => 'Mit giropay zahlen Sie einfach, schnell und sicher im Online-Banking Ihrer teilnehmenden Bank oder Sparkasse. Sie werden direkt zum Online-Banking Ihrer Bank weitergeleitet, wo Sie die Überweisung durch Eingabe von PIN und TAN freigeben.',
        ],
        'en-GB' => [
          'name' => 'giropay - online bank transfer',
          'description' => 'With giropay you pay simply, fast and secure by the online banking of your participating bank. You will be redirected to the online banking of your bank where you authorize the credit transfer.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcgiropay::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //EPS
    $options = [
      'handlerIdentifier' => PaymentGceps::class,
      'id' => $this->getPaymentMethodId(PaymentGceps::class, $context),
      'name' => 'eps - Online Überweisung (AT)',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'eps - Online Überweisung (AT)',
          'description' => 'Mit eps Online-Überweisung zahlen Sie einfach, schnell und sicher im Online-Banking Ihrer Bank. Im nächsten Schritt werden Sie direkt zum Online-Banking Ihrer Bank weitergeleitet, wo Sie die Zahlung durch Eingabe von PIN und TAN freigeben.',
        ],
        'en-GB' => [
          'name' => 'eps - online bank transfer (AT)',
          'description' => 'With eps online transfer you pay simply, fast and secure by online banking of your bank. Your will be redirected to the online banking at your bank where you authorize the credit transfer with PIN and TAN.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGceps::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //IDEAL
    $options = [
      'handlerIdentifier' => PaymentGcideal::class,
      'id' => $this->getPaymentMethodId(PaymentGcideal::class, $context),
      'name' => 'iDEAL - Online überweisen (NL)',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'iDEAL - Online überweisen (NL)',
          'description' => 'iDEAL ist das Online Überweisungssystem der niederländischen Banken, das auf dem etablierten Onlinebanking-Verfahren der Kreditinstitute basiert. Im folgenden müssen Sie Ihre Bank auswählen und können anschließend die Zahlung in Ihrem Online-Banking durchführen.',
        ],
        'en-GB' => [
          'name' => 'iDEAL - online bank transfer (NL)',
          'description' => 'With iDEAL you pay simply, fast and secure by online banking of your bank. You will be redirected to the online banking of your bank where you authorize the credit transfer with PIN and TAN.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcideal::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //MAESTRO
    $options = [
      'handlerIdentifier' => PaymentGcmaestro::class,
      'id' => $this->getPaymentMethodId(PaymentGcmaestro::class, $context),
      'name' => 'Maestro',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'Maestro',
          'description' => 'Maestro ist Ihre Bankkarte für den Geldautomaten, mit der Sie jetzt auch online bezahlen können. Es ist eine Debit-Karte, so dass Sie nie mehr ausgeben können, als Sie auf Ihrem Bankkonto haben.',
        ],
        'en-GB' => [
          'name' => 'Maestro',
          'description' => 'Maestro is your ATM banking card that you can now use to pay online. It\'s a debit facility, so you can never spend more with your Maestro card than you have in your bank account.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcmaestro::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //SOFORTUW
    $options = [
      'handlerIdentifier' => PaymentGcsofortuw::class,
      'id' => $this->getPaymentMethodId(PaymentGcsofortuw::class, $context),
      'name' => 'SOFORT Überweisung',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'SOFORT Überweisung',
          'description' => 'Mit SOFORT Überweisung geht das online Bezahlen einfach wie nie! Sie nutzen dabei ganz bequem Ihre vertrauten Online-Banking Daten und sind so auf der sicheren Seite. Sie werden auf die SOFORT-Webseiten weiter geleitet, geben dort Ihre Bankdaten ein und lösen die Online-Überweisung aus.',
        ],
        'en-GB' => [
          'name' => 'SOFORT Überweisung',
          'description' => 'Direct payment via online banking. Make payments the easy way with your own online banking details – fast and direct. You will be redirected to the web site of SOFORT, where you will enter your banking data and initiate the electronic wire transfer.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcsofortuw::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //BLUECODE
    $options = [
      'handlerIdentifier' => PaymentGcbluecode::class,
      'id' => $this->getPaymentMethodId(PaymentGcbluecode::class, $context),
      'name' => 'Bluecode',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'Bluecode',
          'description' => 'Bluecode funktioniert wie Bargeld – nur auf Ihrem Smartphone. Laden Sie die App jetzt herunter und bezahlen Sie bargeldlos bei über 4500 Partnern. Schnell. Sicher. Einfach.',
        ],
        'en-GB' => [
          'name' => 'Bluecode',
          'description' => 'Bluecode works like cash — just on your smartphone. Download the App now and do cashless payments at over 4500 partners. Fast. Secure. Simple.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcbluecode::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }

    //PAYDIREKT
    $options = [
      'handlerIdentifier' => PaymentGcpaydirekt::class,
      'id' => $this->getPaymentMethodId(PaymentGcpaydirekt::class, $context),
      'name' => 'Giropay/Paydirekt',
      'pluginId' => $pluginId,
      'afterOrderEnabled' => true,
      'translations' => [
        'de-DE' => [
          'name' => 'Giropay/Paydirekt',
          'description' => 'Das Online-Bezahlverfahren Ihrer Banken und Sparkassen. Sie werden auf die Webseiten von Paydirekt weiter geleitet, wo Sie sich mit Ihren persönlichen Zugangsdaten anmelden und dann die Zahlung bestätigen. Sie benötigen vorab eine Registrierung bei Paydirekt.',
        ],
        'en-GB' => [
          'name' => 'Giropay/Paydirekt',
          'description' => 'The online payment method of the German banks and Sparkassen. You will be redirected to the Paydirekt web site, where you will log into your personal account and authorize the payment. You require prior registration with Paydirekt. This payment method only works with German banking accounts.',
        ],],
    ];
    if(!$this->getPaymentMethodId(PaymentGcpaydirekt::class, $context)) {
      $paymentRepository->upsert([$options], $context);
    }
  }

  private function setPaymentMethodIsActive(bool $active, Context $context): void
  {
    $paymentRepository = $this->container->get('payment_method.repository');

    $paymentIds = array(
      PaymentGcbluecode::class,
      PaymentGccreditcard::class,
      PaymentGcdirectdebit::class,
      PaymentGceps::class,
      PaymentGcgiropay::class,
      PaymentGcideal::class,
      PaymentGcmaestro::class,
      PaymentGcpaydirekt::class,
      PaymentGcsofortuw::class,
    );

    foreach ($paymentIds as $class){
      $paymentMethodId = $this->getPaymentMethodId($class, $context);
      // Payment does not even exist, so nothing to (de-)activate here
      if (!$paymentMethodId) {
        return;
      }
      $paymentMethod = [
        'id' => $paymentMethodId,
        'active' => $active,
      ];
      $paymentRepository->update([$paymentMethod], $context);
    }
  }

  private function getPaymentMethodId($paymentClass, $context): ?string
  {
    $paymentRepository = $this->container->get('payment_method.repository');

    // Fetch ID for update
    $paymentCriteria = (new Criteria())->addFilter(new EqualsFilter('handlerIdentifier', $paymentClass));
    $paymentIds = $paymentRepository->searchIds($paymentCriteria, $context);

    if ($paymentIds->getTotal() === 0) {
      return null;
    }
    return $paymentIds->getIds()[0];
  }
}