<?php

namespace GirosGiroCheckout6\Helper;

use GirosGiroCheckout6\GirosGiroCheckout6;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_ResponseCode_helper;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Notify;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Request;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Symfony\Component\HttpFoundation\RedirectResponse;


/**
 * Base class that manage a payment method.
 *
 * @author GiroSolution AG
 * @package GiroCheckout
 * @copyright Copyright (c) 2016, GiroSolution AG
 */
class AbstractPaymentProxy
{
  protected $paymentMethod = '';
  protected $baseUrl = '';
  protected $merchantId = '';
  protected $projectId = '';
  protected $projectPassword = '';
  protected $urlRedirect = '';
  protected $urlNotify = '';
  protected $sendEmail = FALSE;
  protected $createInvoice = FALSE;
  protected $paymentStateSuccess = 12;
  protected $paymentStateFailed = 21;
  protected $paymentStateOpen = 17;
  protected $language = 'de';
  protected $purpose = '';
  protected $useExternalFormService = 0;
  protected $transactionType = 2;

  const GC_TRX_STATE_INDEX = 1;
  const GC_TRX_STATE_NOTIFY = 2;
  const GC_TRX_STATE_RETURN = 3;
  const GC_TRX_STATE_DONE = 4;
  const GC_TRX_STATE_PENDING = 5;

  protected $container;

  /**
   * Constructor method.
   * Load the GiroCheckout library and set the shop language
   *
   * @return void
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function __construct($container)
  {
    $this->container = $container;
    self::includeGiroconnectLibrary();
  }

  /**
   * Set the payment method.
   *
   * @param string $pm The payment method
   * @return string
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   * @package GiroCheckout
   */
  public function setPaymentMethod($pm = '')
  {
    $pm = isset($pm) ? $pm : '';
    $this->paymentMethod = $pm;
  }

  /**
   * Get the payment method.
   *
   * @return string
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getPaymentMethod()
  {
    return $this->paymentMethod;
  }

  /**
   * Set the base URL.
   *
   * @return void
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function setBaseUrl($url = '')
  {
    $url = isset($url) ? (is_string($url) ? trim($url) : '') : '';
    $this->baseUrl = $url;
  }

  /**
   * Get the base URL.
   *
   * @return string
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getBaseUrl()
  {
    return $this->baseUrl;
  }

  /**
   * Get the merchant ID.
   *
   * @return int
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getMerchantId()
  {
    return $this->merchantId;
  }

  /**
   * Get the project ID.
   *
   * @return int
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getProjectId()
  {
    return $this->projectId;
  }

  /**
   * Get the project password.
   *
   * @return string
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getProjectKey()
  {
    return $this->projectKey;
  }

  /**
   * Generate a new transaction ID and store it in the plugin database table.
   *
   * @param object $user User object
   * @return string Generated transaction ID
   */
  public function getUniqueTrxId($userId, $orderId)
  {

    $state = self::GC_TRX_STATE_INDEX;
    $repository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
    $repository->create([
      ['status' => $state,
        'userid' => $userId,
        'orderid' => $orderId,
        'comment' => "",
        'dispatchid' => null,
        'chargevat' => 1]
      ,], Context::createDefaultContext());

    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('userid', $userId));
    $strKey = $repository->search($criteria, Context::createDefaultContext())->last()->get('orderid');
    return $strKey;
  }

  /**
   * Get state for passed transaction and check if it is NOT one of the passed ones.
   *
   * @param int $iTranId Transaction ID
   * @param array $aStates Array of integer constants
   * @return bool TRUE of current state is NOT among the passed ones, FALSE otherwise.
   * @throws \Exception When not found
   */
  public function isStateNot($iTranId, $aStates)
  {
    try {
      $repository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderid', $iTranId));
      $state = $repository->search($criteria, Context::createDefaultContext())->last()->get('status');
      if ($state) {
        return !in_array($state, $aStates);
      };
    } catch (\Exception $e) {
      throw new \Exception("Transaction ID $iTranId not found in GC transactions table");
    }
  }

  /**
   * Set the state for the passed transaction to the passed value.
   *
   * @param int $iTranId Transaction ID
   * @param int $iState State to set to
   * @return bool TRUE on success
   * @throws \Exception When not found
   */
  public function setState($iTranId, $iState, $gcReference = "", $strPayment = "")
  {
    try {
      $repository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderid', $iTranId));
      $trxId = $repository->search($criteria, Context::createDefaultContext())->last()->getId();
      $repository->update([
        ['id' => $trxId,
          'status' => $iState,
          'gcreference' => $gcReference,
          'girocheckout_payment_name' => $strPayment
        ],],
        \Shopware\Core\Framework\Context::createDefaultContext());

      return TRUE;
    } catch (\Exception $e) {
      throw new \Exception("Exception in getUniqueTrxId(), couldn't generate ID");
    }
  }

  public function setPurpose($iTranId, $purpose)
  {
    try {
      $repository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderid', $iTranId));
      $trxId = $repository->search($criteria, Context::createDefaultContext())->last()->getId();
      $repository->update([
        ['id' => $trxId,
          'purpose' => $purpose
        ],],
        \Shopware\Core\Framework\Context::createDefaultContext());

      return TRUE;
    } catch (\Exception $e) {
      throw new \Exception("Exception in getUniqueTrxId(), couldn't generate ID");
    }
  }

  /**
   * Save passed info in transaction table.
   *
   * @param $p_strTranId string Transaction ID to store the info to.
   * @param $p_strOrderId string Order number to insert.
   * @param $p_strGcMandateReference string Order mandete reference number.
   * @return boolean true on success, false on error
   */
  public function saveTransactionData($p_strTranId, $p_strOrderId, $p_strGcReference, $p_strGcMandateReference = "", $p_strGcPaymentName = "")
  {
    try {
      $gcTransId = substr($p_strTranId, 2);  // Remove leading gc
      $repository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('trxid', $gcTransId));
      $trxId = $repository->search($criteria, Context::createDefaultContext())->last()->getId();
      $repository->update([
        [
          'id' => $trxId,
          'gcreference' => $p_strGcReference,
          'girocheckout_directdebit_mandate_reference' => $p_strGcMandateReference,
          'girocheckout_payment_name' => $p_strGcPaymentName
        ],],
        \Shopware\Core\Framework\Context::createDefaultContext());
      return TRUE;
    } catch (\Exception $e) {
      throw new \Exception("Exception in saveTransactionData()");
    }
  }


  /**
   * Save passed info in order attributes table.
   *
   * @param $p_strTranId string Transaction number.
   * @param $p_strGcMandateReference string Order mandete reference number.
   * @return boolean true on success, false on error
   */
  public function saveMandateReference($orderNumber, $mandateReference)
  {
    try {
      $repository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderid', $orderNumber));
      $transaction = $repository->search($criteria, Context::createDefaultContext())->last();
      $update = [
        'id' => $transaction->getId(),
        'girocheckout_directdebit_mandate_reference' => $mandateReference,
      ];
      $repository->update([$update], Context::createDefaultContext());;
    } catch (\Exception $e) {
      error_log($e->getMessage());
    }
    return true;
  }

  /**
   * Get the payment purpose.
   *
   * @return int
   * @throws Exception
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   * @package GiroCheckout
   */
  public function getPurpose($transactionid, $salesChannel)
  {
    $strPurpose = $this->purpose;
    $customerObj = $salesChannel->getCustomer();
    $strShopName = $salesChannel->getSalesChannel()->getName();
    try {
      if (empty($strPurpose)) {
        $strPurpose = "{CUSTOMERID}, {SHOPNAME}";
        $directory = dirname(__FILE__) . "../../../../../../";
        if (file_exists($directory . 'logs')) {
          error_log("Notice Girocheckout: strPurpose is empty in getPurpose() function. " . "\n", 3, $directory . 'logs/gcerror.log');
        } else {
          if (file_exists($directory . 'var/log')) {
            error_log("Notice Girocheckout: strPurpose is empty in getPurpose() function. " . "\n", 3, $directory . 'var/log/gcerror.log');
          } else {
            error_log("Notice Girocheckout: strPurpSose is empty in getPurpose() function. " . "\n", 3, $directory . '/gcerror.log');
          }
        }
      }

      if ($customerObj) {
        $billingAddress = $customerObj->getActiveBillingAddress();
        $iCustomerNr = $customerObj->getCustomerNumber();
        $strLastName = $billingAddress->getLastName();
        $strFirstName = $billingAddress->getFirstName();
        $strName = $billingAddress->getLastName() . ", " . $billingAddress->getFirstName();
      }
      $strPurpose = str_replace("{ORDERID}", $transactionid, $strPurpose);
      $strPurpose = str_replace("{TRANSACTIONID}", $transactionid, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERID}", $iCustomerNr, $strPurpose);
      $strPurpose = str_replace("{SHOPNAME}", $strShopName, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERNAME}", $strName, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERFIRSTNAME}", $strFirstName, $strPurpose);
      $strPurpose = str_replace("{CUSTOMERLASTNAME}", $strLastName, $strPurpose);
    } catch (\Exception $exception) {
      throw new \Exception("Exception in getPurpose(): " . $exception->getMessage());
    }

    if (empty($strPurpose)) {
      if (!empty($strFirstName) || !empty($strLastName)) {
        $strPurpose = "Kunde: " . $strFirstName . " " . $strLastName;
      } else {
        if (!empty($strShopName)) {
          $strPurpose = "Bestellung: " . $strShopName;
        } else {
          $strPurpose = "Bestellung: Shopware";
        }
      }
    }
    return substr($strPurpose, 0, 27);
  }

  /**
   * Set the notify URL.
   *
   * @param string $url The notify URL
   * @return void
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   * @package GiroCheckout
   */
  public function setNotifyUrl($url = '')
  {
    $url = isset($url) ? (is_string($url) ? trim($url) : '') : '';
    $this->urlNotify = $url;
  }

  /**
   * Get the notify URL.
   *
   * @return string
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getNotifyUrl()
  {
    return $this->urlNotify;
  }

  /**
   * Get the redirect URL.
   *
   * @return string
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function setRedirectUrl($url = '')
  {
    $url = isset($url) ? (is_string($url) ? trim($url) : '') : '';
    $this->urlRedirect = $url;
  }

  /**
   * Get the redirect URL.
   *
   * @return string
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getRedirectUrl()
  {
    return $this->urlRedirect;
  }

  /**
   * Get the setting to send mail or not.
   *
   * @return bool
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function canSendEmail()
  {
    return $this->sendEmail;
  }

  /**
   * Get the setting to create an invoice or not.
   *
   * @return bool
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function canCreateInvoice()
  {
    return $this->createInvoice;
  }

  /**
   * Get the setting of payment state success.
   *
   * @return int
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getPaymentStateSuccess()
  {
    return $this->paymentStateSuccess;
  }

  /**
   * Get the setting of payment state failed.
   *
   * @return int
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getPaymentStateFailed()
  {
    return $this->paymentStateFailed;
  }

  /**
   * Get the setting of payment state open.
   *
   * @return int
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public function getPaymentStateOpen()
  {
    return $this->paymentStateOpen;
  }

  /**
   * Get the value and return it as bool.
   *
   * @param bool $value The value to check
   * @return bool
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   * @package GiroCheckout
   */
  public function getAsBoolean($value = FALSE)
  {
    if ($value === 1 || $value === TRUE) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Get the value and return it as int.
   *
   * @param string $value The value to check
   * @return int
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   * @package GiroCheckout
   */
  public function getAsInt($value = '21')
  {
    $result = 21;
    if (isset($value)) {
      if (is_string($value)) {
        $value = trim($value);
        if ($value == '') {
          $value = '21';
        }
      } else {
        if (is_int($value)) {
          $value = trim((string)$value);
          if ($value == '') {
            $value = '21';
          }
        }
      }
    } else {
      $value = '21';
    }

    if ($value != '') {
      if (ctype_digit($value)) {
        $result = intval($value);
        $result = abs($result);
      }
    }
    return $result;
  }

  /**
   * Include the GiroCheckout library.
   *
   * @return void
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @author GiroSolution AG
   */
  public static function includeGiroconnectLibrary()
  {
    $pluginDir = dirname(dirname(__FILE__));
    $libDir = $pluginDir . '/lib/GiroCheckout_SDK';
    $classPath = $libDir . '/GiroCheckout_SDK.php';

    if (is_file($classPath)) {
      include_once $classPath;
    }
  }

  public function getGcSource()
  {
    return "Shopware ".$this->container->getParameter('kernel.shopware_version')." ;Shopware Plugin " .GirosGiroCheckout6::PLUGIN_VERSION;
  }

  /**
   * Handles notification.
   *
   * @param array $aParams Notification array
   * @param string $p_strTransName Name of transaction type for SDK
   * @param string $p_strPwd Project password
   * @param string $p_strPmtName Name of payment method for trx table
   * @param string $p_strTransType Transaction type of payment
   * @throws \Exception
   */
  public function notifyAction($aParams, $p_strTransName, $p_strPwd, $p_strPmtName, $p_strTransType, $transactionStateHandler)
  {
    $notify = new GiroCheckout_SDK_Notify($p_strTransName);
    try {

      $notify->setSecret(trim((string)$p_strPwd));
      $notify->parseNotification($aParams);
      $iTranId = $notify->getResponseParam('gcMerchantTxId');
      $gctrxrepository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
      $orderRepository = $this->container->get('order.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderNumber', $iTranId))
        ->addAssociation('transactions');
      $order = $orderRepository->search($criteria, Context::createDefaultContext())->last();
      $iOrderId = $order->getId();
      $transactionRepository = $this->container->get('order_transaction.repository');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderId', $iOrderId));
      $transaction = $transactionRepository->search($criteria, Context::createDefaultContext())->last();
      $transactionState = $transaction->getStateMachineState();

      //Process States
      if ($this->isStateNot($iTranId, array(self::GC_TRX_STATE_INDEX))){
        // Transaction already processed
        $notify->sendOkStatus();
        $notify->setNotifyResponseParam('Result', 'OK');
        $notify->setNotifyResponseParam('ErrorMessage', 'Notify detected transaction already processed');
        $notify->setNotifyResponseParam('MailSent', '');
        $notify->setNotifyResponseParam('OrderId', $iTranId);
        $notify->setNotifyResponseParam('CustomerId', "");
        return $notify->getNotifyResponseStringJson();
      }
      else {
        // Process transaction here
        $this->setState($iTranId, self::GC_TRX_STATE_NOTIFY, $notify->getResponseParam('gcReference'), $p_strPmtName);

        // Checks if the payment was successful
        if ($notify->paymentSuccessful()) {

          // Check Refundable payments
          $bIsCapturable = "0";
          $bIsRefundable = "0";

          if ($p_strTransType == "AUTH") {
            if ($p_strPmtName == 'gc_creditcard' ||
              $p_strPmtName == 'gc_directdebit' ||
              $p_strPmtName == 'gc_maestro' ||
              $p_strPmtName == 'gc_bluecode' ||
              $p_strPmtName == 'gc_paydirekt') {

              $bIsCapturable = "1";
            }
          }
          else {
            if ($p_strPmtName == 'gc_creditcard' ||
              $p_strPmtName == 'gc_directdebit' ||
              $p_strPmtName == 'gc_maestro' ||
              $p_strPmtName == 'gc_bluecode' ||
              $p_strPmtName == 'gc_paydirekt') {

              $bIsRefundable = "1";
            }
          }

          // Update Order Custom Fields
          $customFields = [
            'pluginName' => 'gsgirocheckout',
            'paymentName' => $p_strPmtName,
            'isRefundable' => $bIsRefundable,
            'isCapturable' => $bIsCapturable,
            'gcReference' => $notify->getResponseParam('gcReference'),
            'orderNumber' => $iTranId,
            'transactionType' => $p_strTransType,
          ];

          // Create extra custom field for direct debit payment.
          if ($p_strPmtName == 'gc_directdebit') {

            // Search for mandate reference if payment is direct debit
            $criteria = new Criteria();
            $criteria->addFilter(new EqualsFilter('orderid', $iTranId));
            $mandateReference = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->get('girocheckout_directdebit_mandate_reference');

            $customFields = [
              'pluginName' => 'gsgirocheckout',
              'paymentName' => $p_strPmtName,
              'isRefundable' => $bIsRefundable,
              'isCapturable' => $bIsCapturable,
              'gcReference' => $notify->getResponseParam('gcReference'),
              'orderNumber' => $iTranId,
              'transactionType' => $p_strTransType,
              'mandateReference' => $mandateReference,
            ];
          }

          $update = [
            'id' => $transaction->getId(),
            'customFields' => $customFields,
          ];

          // Update order data in database.
          $transactionRepository->update([$update], Context::createDefaultContext());

          //Check if transaction is marked as Cancelled.
          if($transactionState->getTechnicalName() === "cancelled"){
            $transactionStateHandler->reopen($transaction->getId(), Context::createDefaultContext());
            $transactionStateHandler->process($transaction->getId(), Context::createDefaultContext());
          }

          //Pay Transaction.
          if (method_exists($transactionStateHandler, 'paid')) {
            $transactionStateHandler->paid($transaction->getId(), Context::createDefaultContext());
          }
          else {
            $transactionStateHandler->pay($transaction->getId(), Context::createDefaultContext());
          }

          $notify->sendOkStatus();
          $notify->setNotifyResponseParam('Result', 'OK');
          $notify->setNotifyResponseParam('ErrorMessage', '');
          $notify->setNotifyResponseParam('MailSent', '');
          $notify->setNotifyResponseParam('OrderId', $iOrderId);
          $notify->setNotifyResponseParam('CustomerId', '');//$trxdata['userid']);
          $this->setState($iTranId, self::GC_TRX_STATE_DONE, $notify->getResponseParam('gcReference'), $p_strPmtName);
          return $notify->getNotifyResponseStringJson();
        } else {

          //Check if user canceled the order
          if($notify->getResponseParam('gcResultPayment') == 4502){
            $transactionStateHandler->cancel($transaction->getId(), Context::createDefaultContext());
          }
          $notify->sendOkStatus();
          $notify->setNotifyResponseParam('Result', 'OK');
          $notify->setNotifyResponseParam('ErrorMessage', '');
          $notify->setNotifyResponseParam('MailSent', '');
          $notify->setNotifyResponseParam('OrderId', '');  // Order not created
          $notify->setNotifyResponseParam('CustomerId', '');//$trxdata['userid']);
          $this->setState($iTranId, self::GC_TRX_STATE_PENDING, $notify->getResponseParam('gcReference'), $p_strPmtName);
          return $notify->getNotifyResponseStringJson();
        }
      }
    } catch (\Exception $e) {
      $notify->sendBadRequestStatus();
      $notify->setNotifyResponseParam('Result', 'ERROR');
      $notify->setNotifyResponseParam('ErrorMessage', $e->getMessage());
      return $notify->getNotifyResponseStringJson();
    }
  }
}
