<?php

namespace GirosGiroCheckout6\Helper;

use GirosGiroCheckout6\GirosGiroCheckout6;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Request;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_ResponseCode_helper;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Checkout\Test\Cart\Common\Generator;
use Shopware\Core\Defaults;

/**
 * Class to manage a BlueCode payment method.
 *
 * Extends the AbstractPaymentProxy
 *
 * @author GiroSolution AG
 * @package GiroCheckout
 * @copyright Copyright (c) 2016, GiroSolution AG
 */
class BluecodePaymentProxy extends AbstractPaymentProxy
{

  private static $refINSTANCE = NULL;
  protected $container;
  protected $config;

  /**
   * Constructor method.
   * Set the payment type and payment method credentials.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return void
   */
  public function __construct($container, $systemConfigService)
  {
    $this->container = $container;
    $this->config = $systemConfigService;
    parent::__construct($container);
    self::$refINSTANCE = $this;
    $this->setPaymentMethod("gc_bluecode");
    $this->pReadConfiguration();
  }

  /**
   * Process the payment transaction.
   * In this section a request method instance has to be passed and then is submits the request to the GiroCheckout API.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @param int $amount The amount of order.
   * @param string $currency The shop currency.
   * @param string $transactionId The order number.
   * @param string $text The payment purpose.
   * @param string $cardtype The issuer selected.
   * @return array
   */
  public function startTransaction($amount = '', $currency = '', $transactionId = '', $text = '', $salesChannelContext = '')
  {
    $language = "de";
    $amount = isset($amount) ? (is_string($amount) ? trim($amount) : '') : '';
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $result = array();

    try {
      $request = new GiroCheckout_SDK_Request('blueCodeTransaction');
      $request->setSecret($this->projectPassword);
      $request->addParam('merchantId', $this->merchantId)
          ->addParam('projectId', $this->projectId)
          ->addParam('merchantTxId', $transactionId)
          ->addParam('amount', $amount * 100)
          ->addParam('currency', $currency)
          ->addParam('purpose', $text)
          ->addParam('urlRedirect', $this->urlRedirect)
          ->addParam('urlNotify', $this->urlNotify)
          ->addParam('sourceId', $this->getGcSource())
          ->addParam('customerId', $salesChannelContext->getCustomer()->getId())
          ->submit();

      if ($request->requestHasSucceeded()) {
        $strUrlRedirect = $request->getResponseParam('redirect');
        $result["status"] = $request->getResponseParam('rc');
        $result["redirect"] = $strUrlRedirect;
      } else {
        $result["status"] = $request->getResponseParam('rc');
        $result["msg"] = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $language);
      }
    } catch (\Exception $e) {
      $result["status"] = -1000;
      $result["msg"] = $e->getMessage();
    }
    return $result;
  }

  /**
   * Read the payment method credentials values from database.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return void
   */
  private function pReadConfiguration()
  {
    $config = $this->config->get('GirosGiroCheckout6.config');
    if (isset($config)) {
      if (isset($config['gcbluecodeMerchantId'])) {
        $this->merchantId = trim((string)$config['gcbluecodeMerchantId']);
      }

      if (isset($config['gcbluecodeProjectId'])) {
        $this->projectId = trim((string)$config['gcbluecodeProjectId']);
      }

      if (isset($config['gcbluecodeProjectPassword'])) {
        $this->projectPassword = trim((string)$config['gcbluecodeProjectPassword']);
      }

      if (isset($config['gcbluecodePurpose'])) {
        $this->purpose = trim((string)$config['gcbluecodePurpose']);
      }

      // PaymentStateSuccess
      if (isset($config['gcbluecodePaymentStateSuccess'])) {
        $this->paymentStateSuccess = $this->getAsInt($config['gcbluecodePaymentStateSuccess']);
      }

      // Email senden, wenn sich der Bezahlstatus aendert
      if (defined('GIROCONNECT_CONFIGURE_SEND_EMAIL') && GIROCONNECT_CONFIGURE_SEND_EMAIL === TRUE) {
        if (isset($config['gcbluecodeStatusMail'])) {
          $value = $this->getAsBoolean($config['gcbluecodeStatusMail']);
          $this->sendEmail = $value;
        }
      }
    }
  }

  public function capturePayment($amount = '', $currency = '', $transactionId = '', $reference = '', $userid = '')
  {
    $amount = strval($amount);
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $merchantId = $this->merchantId;
    $projectId = $this->projectId;
    $password = $this->projectPassword;

    try {
      $orderRepository = $this->container->get('order.repository');
      $transactionRepository = $this->container->get('order_transaction.repository');
      $gctrxrepository = $this->container->get('gc_user_pay_giroconnect_trx.repository');

      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
      $purpose = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->get('purpose');

      $request = new GiroCheckout_SDK_Request('blueCodeCapture');
      $request->setSecret($password);
      $request->addParam('merchantId', $merchantId)
          ->addParam('projectId', $projectId)
          ->addParam('merchantTxId', $transactionId)
          ->addParam('amount', $amount)
          ->addParam('currency', $currency)
          ->addParam('reference', $reference)
          ->submit();
      if ($request->requestHasSucceeded()) {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
        $order = $orderRepository->search($criteria, Context::createDefaultContext())->last();
        $iOrderId = $order->getId();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $iOrderId));
        $transaction = $transactionRepository->search($criteria, Context::createDefaultContext())->last();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
        $trxId = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->getId();
        $gctrxrepository->update([
            ['id' => $trxId,
                'girocheckout_capture_reference' => $request->getResponseParam('reference')
            ],], Context::createDefaultContext());


        // Update order data in database.
        $customFields = [
            'pluginName' => 'gsgirocheckout',
            'paymentName' => 'gc_bluecode',
            'isRefundable' => '1',
            'isCapturable' => '0',
            'gcReference' => $request->getResponseParam('reference'),
            'orderNumber' => $transactionId,
            'transactionType' => 'AUTH',
        ];

        $update = [
            'id' => $transaction->getId(),
            'customFields' => $customFields,
        ];

        $transactionRepository->update([$update], Context::createDefaultContext());

        $return["status"] = "OK";
        $return["message"] = "Payment of " . $transactionId . " was captured - Ref. ID:" . $reference;
        $return["reference"] = $request->getResponseParam('reference');
      } else {
        $return["status"] = "ERROR";
        $return["message"] = "Payment could not captured:" . $request->getResponseMessage($request->getResponseParam('rc'), 'DE');
        $return["reference"] = $reference;
      }
    } catch (\Exception $e) {
      $return["status"] = "ERROR";
      $return["message"] = "Error in capture: " . $e->getMessage();
      $return["reference"] = $reference;

    }
    return $return;
  }

  public function processRefund($amount = '', $currency = '', $transactionId = '', $reference = '', $userid = '')
  {
    $amount = strval($amount);
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $merchantId = $this->merchantId;
    $projectId = $this->projectId;
    $password = $this->projectPassword;

    $gctrxrepository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
    $transactionRepository = $this->container->get('order_transaction.repository');
    $orderRepository = $this->container->get('order.repository');

    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
    $purpose = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->get('purpose');

    try {
      $request = new GiroCheckout_SDK_Request('blueCodeRefund');
      $request->setSecret($password);
      $request->addParam('merchantId', $merchantId)
          ->addParam('projectId', $projectId)
          ->addParam('merchantTxId', $transactionId)
          ->addParam('amount', $amount)
          ->addParam('currency', $currency)
          ->addParam('reference', $reference)
          ->addParam('purpose', $purpose)
          ->submit();

      if ($request->requestHasSucceeded()) {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $merchantId));

        // Update order data in database.
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
        $order = $orderRepository->search($criteria, Context::createDefaultContext())->last();
        $iOrderId = $order->getId();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $iOrderId));
        $transaction = $transactionRepository->search($criteria, Context::createDefaultContext())->last();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
        $trxId = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->getId();
        $gctrxrepository->update([
            ['id' => $trxId,
                'girocheckout_refund_reference' => $request->getResponseParam('reference')
            ],], Context::createDefaultContext());

        $customFields = [
            'pluginName' => 'gsgirocheckout',
            'paymentName' => 'gc_bluecode',
            'isRefundable' => '0',
            'isCapturable' => '0',
            'gcReference' => $request->getResponseParam('reference'),
            'orderNumber' => $merchantId,
            'transactionType' => 'AUTH',
        ];

        $update = [
            'id' => $transaction->getId(),
            'customFields' => $customFields,
        ];

        $transactionRepository->update([$update], Context::createDefaultContext());

        $return["status"] = "OK";
        $return["message"] = "Payment of " . $transactionId . " was captured - Ref. ID:" . $reference;
        $return["reference"] = $request->getResponseParam('reference');
      } else {
        $return["message"] = "Payment could not refunded:" . $request->getResponseMessage($request->getResponseParam('rc'), 'DE');
        $return["reference"] = $reference;
        $return["status"] = "ERROR";
      }
    } catch (\Exception $e) {
      $return["status"] = "ERROR";
      $return["message"] = "Error in refund: " . $e;
      $return["reference"] = $reference;
    }
    return $return;
  }
}
