<?php

namespace GirosGiroCheckout6\Helper;

use GirosGiroCheckout6\GirosGiroCheckout6;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Request;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_ResponseCode_helper;
use girosolution\GiroCheckout_SDK\GiroCheckout_SDK_Tools;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Checkout\Test\Cart\Common\Generator;
use Shopware\Core\Defaults;




/**
 * Class to manage a creditcard payment method.
 *
 * Extends the AbstractPaymentProxy
 *
 * @author GiroSolution AG
 * @package GiroCheckout
 * @copyright Copyright (c) 2016, GiroSolution AG
 */
class CreditcardPaymentProxy extends AbstractPaymentProxy
{
  private static $refINSTANCE = NULL;
  protected $UseVisaMaster = false;
  protected $UseAMEX = false;
  protected $UseJCB = false;
  protected $container;
  protected $config;


  /**
   * Constructor method.
   * Set the payment type and payment method credentials.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return void
   */
  public function __construct($container, $systemConfigService)
  {
    $this->container = $container;
    $this->config = $systemConfigService;
    parent::__construct($container);
    self::$refINSTANCE = $this;
    $this->setPaymentMethod("gc_creditcard");
    $this->pReadConfiguration();
  }


  /**
   * Get the value to use the VisaMaster creditcard.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return int
   */
  public function getUseVisaMaster()
  {
    if (!isset($this->UseVisaMaster) || empty($this->UseVisaMaster))
      return false;
    else
      return true;
  }

  /**
   * Get the value to use the AMEX creditcard.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return int
   */
  public function getUseAmex()
  {
    if (!isset($this->UseAMEX) || empty($this->UseAMEX))
      return false;
    else
      return true;
  }

  /**
   * Get the value to use the JCB creditcard.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return int
   */
  public function getUseJCB()
  {
    if (!isset($this->UseJCB) || empty($this->UseJCB))
      return false;
    else
      return true;
  }

  /**
   * Get the extended logo for creditcards.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return string
   */
  public function getExtendedLogo()
  {
    $visa_msc = $this->UseVisaMaster;
    $amex = $this->UseAMEX;
    $jcb = $this->UseJCB;

    if (!$visa_msc && !$amex && !$jcb)
      $visa_msc = true;

    return GiroCheckout_SDK_Tools::getCreditCardLogoName($visa_msc, $amex, $jcb);
  }

  /**
   * Process the payment transaction.
   * In this section a request method instance has to be passed and then is submits the request to the GiroCheckout API.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @param int $amount The amount of order.
   * @param string $currency The shop currency.
   * @param string $transactionId The order number.
   * @param string $text The payment purpose.
   * @return array
   */
  public function startTransaction($amount = '', $currency = '', $transactionId = '', $purpose = '', $salesChannelContext = '')
  {
    $language = "de";
    $amount = isset($amount) ? (is_string($amount) ? trim($amount) : '') : '';
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $result = array();
    $addressesMatch = false;
    $customer = $salesChannelContext->getCustomer();
    $billingAddress = $salesChannelContext->getCustomer()->getActiveBillingAddress();
    $shippingAddress = $salesChannelContext->getCustomer()->getActiveShippingAddress();

    //Check if Billing and Shipping are the same.
    if($billingAddress->getId() == $shippingAddress->getId()){
      $addressesMatch = true;
    }

    //Set Data
    $tds2Address = $shippingAddress->getStreet();
    $tds2Postcode = $shippingAddress->getZipcode();
    $tds2City = $shippingAddress->getCity();
    $tds2Country = $shippingAddress->getCountry()->getIso();
    $aTdsOptionalInfo = new \stdClass;
    $aTdsOptionalInfo->email = $customer->getEmail();
    $aTdsOptionalInfo->addressesMatch = "true";

    if(!$addressesMatch){
      $tds2Address = $billingAddress->getStreet();
      $tds2Postcode = $billingAddress->getZipcode();
      $tds2City = $billingAddress->getCity();
      $tds2Country = $billingAddress->getCountry()->getIso();
      $aTdsOptionalInfo->addressesMatch = "false";


      if(!empty($shippingAddress->getStreet()) && !empty($shippingAddress->getZipcode())
        && !empty($shippingAddress->getCity()) && !empty($shippingAddress->getCountry()->getIso()))
      {
        $aTdsOptionalInfo->shippingAddress = new \stdClass;
        $aTdsOptionalInfo->shippingAddress->line1 = $shippingAddress->getStreet();
        $aTdsOptionalInfo->shippingAddress->postcode = $shippingAddress->getZipcode();
        $aTdsOptionalInfo->shippingAddress->city = $shippingAddress->getCity();
        $aTdsOptionalInfo->shippingAddress->country = $shippingAddress->getCountry()->getIso();
      }
    }

    try {
      $request = new GiroCheckout_SDK_Request('creditCardTransaction');
      $request->setSecret($this->projectPassword);
      $request->addParam('merchantId', $this->merchantId)
          ->addParam('projectId', $this->projectId)
          ->addParam('merchantTxId', $transactionId)
          ->addParam('amount', $amount * 100)
          ->addParam('currency', $currency)
          ->addParam('purpose', $purpose)
          ->addParam('locale', $language)
          ->addParam('tds2Address', $tds2Address)
          ->addParam('tds2Postcode',$tds2Postcode)
          ->addParam('tds2City',$tds2City)
          ->addParam('tds2Country',$tds2Country)
          ->addParam('tds2Optional',json_encode($aTdsOptionalInfo))
          ->addParam('urlRedirect', $this->urlRedirect)
          ->addParam('urlNotify', $this->urlNotify)
          ->addParam('sourceId',$this->getGcSource())
          ->addParam('customerId', $salesChannelContext->getCustomer()->getId())
          ->addParam('type', $this->transactionType)
          ->submit();

      if ($request->requestHasSucceeded()) {
        $strUrlRedirect = $request->getResponseParam('redirect');
        $result["status"] = $request->getResponseParam('rc');
        $result["redirect"] = $strUrlRedirect;
      } else {
        $result["msg"] = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $language);
      }
    } catch (\Exception $e) {
      $result["msg"] = $e->getMessage();
    }
    return $result;
  }

  /**
   * Read the payment method credentials values from database.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2016, GiroSolution AG
   * @return void
   */
  private function pReadConfiguration()
  {
    $config = $this->config->get('GirosGiroCheckout6.config');
    if (isset($config)) {
      if (isset($config['gccreditcardMerchantId'])) {
        $this->merchantId = trim((string)$config['gccreditcardMerchantId']);
      }
//
      if (isset($config['gccreditcardProjectId'])) {
        $this->projectId = trim((string)$config['gccreditcardProjectId']);
      }
//
      if (isset($config['gccreditcardProjectPassword'])) {
        $this->projectPassword = trim((string)$config['gccreditcardProjectPassword']);
      }
//
      if (isset($config['gccreditcardPurpose'])) {
        $this->purpose = trim((string)$config['gccreditcardPurpose']);
      }

//
      // Transaction Type
      if (isset($config['gccreditcardTransactionType'])) {
        $this->transactionType = trim((string)$config['gccreditcardTransactionType']);
      }

      // gccreditcardVisaMaster
      if (isset($config['gccreditcardVisaMaster'])) {
        $this->UseVisaMaster = $this->getAsBoolean($config['gccreditcardVisaMaster']);
      }
//
      // gccreditcardAmex
      if (isset($config['gccreditcardAmex'])) {
        $this->UseAMEX = $this->getAsBoolean($config['gccreditcardAmex']);
      }

      // gccreditcardJCB
      if (isset($config['gccreditcardJCB'])) {
        $this->UseJCB = $this->getAsBoolean($config['gccreditcardJCB']);
      }
//
//
      // Email senden, wenn sich der Bezahlstatus aendert
      if (defined('GIROCONNECT_CONFIGURE_SEND_EMAIL') && GIROCONNECT_CONFIGURE_SEND_EMAIL === TRUE) {
        if (isset($config['gccreditcardStatusMail'])) {
          $value = $this->getAsBoolean($config['gccreditcardStatusMail']);
          $this->sendEmail = $value;
        }
      }
    }
  }

  public function capturePayment($amount = '', $currency = '', $transactionId = '', $reference = '', $userid = '')
  {
    $amount = strval($amount);
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $merchantId = $this->merchantId;
    $projectId = $this->projectId;
    $password = $this->projectPassword;

    try {
      $orderRepository = $this->container->get('order.repository');
      $transactionRepository = $this->container->get('order_transaction.repository');
      $gctrxrepository = $this->container->get('gc_user_pay_giroconnect_trx.repository');

      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
      $criteria = new Criteria();
      $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
      $purpose = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->get('purpose');

      $request = new GiroCheckout_SDK_Request('creditCardCapture');
      $request->setSecret($password);
      $request->addParam('merchantId', $merchantId)
        ->addParam('projectId', $projectId)
        ->addParam('merchantTxId', $transactionId)
        ->addParam('amount', $amount)
        ->addParam('currency', $currency)
        ->addParam('reference', $reference)
        ->submit();
      if ($request->requestHasSucceeded()) {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
        $order = $orderRepository->search($criteria, Context::createDefaultContext())->last();
        $iOrderId = $order->getId();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $iOrderId));
        $transaction = $transactionRepository->search($criteria, Context::createDefaultContext())->last();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
        $trxId = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->getId();
        $gctrxrepository->update([
          ['id' => $trxId,
            'girocheckout_capture_reference' => $request->getResponseParam('reference')
          ],], Context::createDefaultContext());


        // Update order data in database.
        $customFields = [
          'pluginName' => 'gsgirocheckout',
          'paymentName' => 'gc_creditcard',
          'isRefundable' => '1',
          'isCapturable' => '0',
          'gcReference' => $request->getResponseParam('reference'),
          'orderNumber' => $transactionId,
          'transactionType' => 'AUTH',
        ];

        $update = [
          'id' => $transaction->getId(),
          'customFields' => $customFields,
        ];

        $transactionRepository->update([$update], Context::createDefaultContext());

        $return["status"] = "OK";
        $return["message"] = "Payment of " . $transactionId . " was captured - Ref. ID:" . $reference;
        $return["reference"] = $request->getResponseParam('reference');
      } else {
        $return["status"] = "ERROR";
        $return["message"] = "Payment could not captured:" . $request->getResponseMessage($request->getResponseParam('rc'), 'DE');
        $return["reference"] = $reference;
      }
    } catch (\Exception $e) {
      $return["status"] = "ERROR";
      $return["message"] = "Error in capture: " . $e;
      $return["reference"] = $reference;

    }
    return $return;
  }

  public function processRefund($amount = '', $currency = '', $transactionId = '', $reference = '', $userid = '')
  {
    $amount = strval($amount);
    $currency = isset($currency) ? (is_string($currency) ? trim($currency) : 'EUR') : 'EUR';
    $transactionId = isset($transactionId) ? (is_string($transactionId) ? trim($transactionId) : '') : '';
    $merchantId = $this->merchantId;
    $projectId = $this->projectId;
    $password = $this->projectPassword;

    $gctrxrepository = $this->container->get('gc_user_pay_giroconnect_trx.repository');
    $transactionRepository = $this->container->get('order_transaction.repository');
    $orderRepository = $this->container->get('order.repository');

    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
    $purpose = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->get('purpose');

    try {
      $request = new GiroCheckout_SDK_Request('creditCardRefund');
      $request->setSecret($password);
      $request->addParam('merchantId', $merchantId)
        ->addParam('projectId', $projectId)
        ->addParam('merchantTxId', $transactionId)
        ->addParam('amount', $amount)
        ->addParam('currency', $currency)
        ->addParam('reference', $reference)
        ->addParam('purpose', $purpose)
        ->submit();

      if ($request->requestHasSucceeded()) {

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $merchantId));

        // Update order data in database.

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderNumber', $transactionId))->addAssociation('transactions');
        $order = $orderRepository->search($criteria, Context::createDefaultContext())->last();
        $iOrderId = $order->getId();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderId', $iOrderId));
        $transaction = $transactionRepository->search($criteria, Context::createDefaultContext())->last();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('orderid', $transactionId));
        $trxId = $gctrxrepository->search($criteria, Context::createDefaultContext())->last()->getId();
        $gctrxrepository->update([
          ['id' => $trxId,
            'girocheckout_refund_reference' => $request->getResponseParam('reference')
          ],], Context::createDefaultContext());

        $customFields = [
          'pluginName' => 'gsgirocheckout',
          'paymentName' => 'gc_creditcard',
          'isRefundable' => '0',
          'isCapturable' => '0',
          'gcReference' => $request->getResponseParam('reference'),
          'orderNumber' => $merchantId,
          'transactionType' => 'AUTH',
        ];

        $update = [
          'id' => $transaction->getId(),
          'customFields' => $customFields,
        ];

        $transactionRepository->update([$update], Context::createDefaultContext());

        $return["status"] = "OK";
        $return["message"] = "Payment of " . $transactionId . " was captured - Ref. ID:" . $reference;
        $return["reference"] = $request->getResponseParam('reference');
      } else {
        $return["message"] = "Payment could not refunded:" . $request->getResponseMessage($request->getResponseParam('rc'), 'DE');
        $return["reference"] = $reference;
        $return["status"] = "ERROR";
      }
    } catch (\Exception $e) {
      $return["status"] = "ERROR";
      $return["message"] = "Error in refund: " . $e;
      $return["reference"] = $reference;
    }
    return $return;
  }
}