<?php declare(strict_types=1);

namespace GirosGiroCheckout6\Service;

use Exception;
use GirosGiroCheckout6\Helper\IdealPaymentProxy;
use GirosGiroCheckout6\Storefront\Controller\NotificationController;
use GirosGiroCheckout6\Storefront\Controller\ReturnController;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentFinalizeException;
use Shopware\Core\Checkout\Payment\Exception\CustomerCanceledAsyncPaymentException;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use girosolution\GiroCheckout_SDK\helper\GiroCheckout_SDK_ResponseCode_helper;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;


/**
 * Shopware Payment Service for creditcard.
 *
 * Implements the AsynchronousPaymentHandlerInterface
 *
 * @author GiroSolution AG
 * @package GiroCheckout
 * @copyright Copyright (c) 2016, GiroSolution AG
 */
class PaymentGcideal implements AsynchronousPaymentHandlerInterface
{
  private $transactionStateHandler;
  private $container;
  private $notificationController;
  private $systemConfigService;
  private $returnController;



  public function __construct(OrderTransactionStateHandler $transactionStateHandler,
                              ContainerInterface $container,
                              NotificationController $notificationController,
                              SystemConfigService $systemConfigService,
                              ReturnController $returnController)
  {

    $this->transactionStateHandler = $transactionStateHandler;
    $this->container = $container;
    $this->notificationController = $notificationController;
    $this->systemConfigService = $systemConfigService;
    $this->returnController = $returnController;
  }

  public function pay(AsyncPaymentTransactionStruct $transaction,
                      RequestDataBag $dataBag,
                      SalesChannelContext $salesChannelContext): RedirectResponse
  {
    try {
      $paymentProxy = new IdealPaymentProxy($this->container, $this->systemConfigService);
      $notifyUrl = $this->notificationController->getNotifyUrl('gc_ideal');
      $redirectUrl = $this->returnController->getReturnUrl('gc_ideal');
      $paymentProxy->setBaseUrl("");
      $paymentProxy->setRedirectUrl($redirectUrl);
      $paymentProxy->setNotifyUrl($notifyUrl);
      $amount = (string)round($transaction->getOrder()->getAmountTotal(), 2);
      $currency = $salesChannelContext->getCurrency()->getIsoCode();
      $transactionId = $paymentProxy->getUniqueTrxId($salesChannelContext->getCustomer()->getId(), $transaction->getOrder()->getOrderNumber());
      $purpose = $paymentProxy->getPurpose($transactionId, $salesChannelContext);
      $startTransactionResult = $paymentProxy->startTransaction($amount, $currency, $transactionId, $purpose, $salesChannelContext);
      if ($startTransactionResult['status'] != 0) {
        $errorMsg = $startTransactionResult['msg'];
        $errorUrl= $this->returnController->getErrorUrl($errorMsg);
        return new RedirectResponse($errorUrl);
      } else {
        if (isset($startTransactionResult['redirect'])) {
          // Redirect to external gateway
          return new RedirectResponse($startTransactionResult['redirect']);
        } else {
          $errorMsg = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, "de");
          $errorUrl= $this->returnController->getErrorUrl($errorMsg);
          return new RedirectResponse($errorUrl);
        }
      }
    } catch (\Exception $e) {
      throw new AsyncPaymentProcessException(
          $transaction->getOrderTransaction()->getId(), 'An error occurred during the communication with external payment gateway' . PHP_EOL . $e->getMessage());
    }
  }


  public function finalize(AsyncPaymentTransactionStruct $transaction,
                           Request $request,
                           SalesChannelContext $salesChannelContext): void
  {
    try {
      $context = $salesChannelContext->getContext();
      $orderTransactionId = $transaction->getOrderTransaction()->getId();
      $orderId = $transaction->getOrder()->getOrderNumber();
      $paymentStatusCode = $request->query->get('gcResultPayment');
      $gcTransId = $request->query->get('gcMerchantTxId');
      $gcRepository = $this->container->get('gc_user_pay_giroconnect_trx.repository');


      if ($orderId !== (string)$gcTransId) {
        throw new Exception('Order number mismatch.');
      }

      //Check Response Code
      if ($paymentStatusCode != 4000) {
        $msg = GiroCheckout_SDK_ResponseCode_helper::getMessage($paymentStatusCode, "DE");
        $this->transactionStateHandler->cancel($transaction->getOrderTransaction()->getId(), $context);
        throw new CustomerCanceledAsyncPaymentException($orderTransactionId, 'Error: ' . $msg);
      }

    } catch (Exception $exception) {
      throw new AsyncPaymentFinalizeException($orderTransactionId, $exception->getMessage());
    }
  }
}
